using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.IO;
using System.Threading;

using Org.Shokai.Gis.Gps;
using Org.Shokai.Template;
using EARTHLib;

namespace GPS_CS1KReader
{
    public partial class Form1 : Form
    {
        private SonyCs1kReader gpsReader;
        private IApplicationGE gEarth;
        public ArrayList dataListList;
        private Simplate template;
        private DirectoryInfo gps_cs1k_dir;

        public Form1()
        {
            InitializeComponent();

            gps_cs1k_dir = new DirectoryInfo(
                new DirectoryInfo(Application.UserAppDataPath).Parent.Parent.Parent.FullName
                + @"\Sony Corporation\Sony Picture Utility\GPSUtil\Log");
            if(gps_cs1k_dir.Exists) textBoxDirPath.Text = gps_cs1k_dir.FullName;
        }

        private void buttonLoad_Click(object sender, EventArgs e)
        {
            gEarth = new ApplicationGE();

            gpsReader = new SonyCs1kReader();
            dataListList = gpsReader.ReadDirectory(new DirectoryInfo(textBoxDirPath.Text));
            trace("GPS Data Load ("+dataListList.Count+"files)");

            template = new Simplate();
            template.TemplateDirectory = new DirectoryInfo(
                new DirectoryInfo(Application.StartupPath).Parent.Parent.FullName + @"\template");
            template.LoadTemplateFile("walkpath.kml");

            Hashtable hash = new Hashtable();
            hash["name"] = "Sony GPS-CS1K Tracker";
            hash["description"] = DateTime.Today.ToString("yyyy/MM/dd") + " Update";

            ArrayList lines = new ArrayList();
            for (int i = 0; i < dataListList.Count; i++)
            {
                GpsDataList dataList = (GpsDataList)dataListList[i];
                Hashtable line = new Hashtable();
                line["line_name"] = ((GpsData)dataList[0]).time.ToString("yyyy/MM/dd HH:mm:ss");
                line["line_description"] = "~" + ((GpsData)dataList[dataList.Count - 1]).time.ToString("yyyy/MM/dd HH:mm:ss");
                line["line_style"] = "walkpath";
                foreach (GpsData gpsData in dataList)
                {
                    line["line_coordinates"] += gpsData.lon + "," + gpsData.lat + " ";
                }
                lines.Add(line);

                if (i < dataListList.Count - 1)
                {
                    line = new Hashtable();
                    line["line_style"] = "jumppath";
                    GpsData jumpFrom, jumpTo;
                    jumpFrom = (GpsData)((GpsDataList)dataListList[i])[((GpsDataList)dataListList[i]).Count - 1];
                    jumpTo = (GpsData)((GpsDataList)dataListList[i + 1])[0];
                    line["line_coordinates"] += jumpFrom.lon + "," + jumpFrom.lat + " " + jumpTo.lon + "," + jumpTo.lat;
                    line["line_name"] = "(jump) " + jumpFrom.time.ToString("yyyy/MM/dd HH:mm:ss");
                    line["line_description"] = "~" + jumpTo.time.ToString("yyyy/MM/dd HH:mm:ss");
                    lines.Add(line);
                }
            }
            hash["[lines]"] = lines;

            template.Process(hash);

            template.SaveResultFile("result-walkpath.kml");
            gEarth.OpenKmlFile(Application.StartupPath + @"\result-walkpath.kml", 0);
            trace("GoogleEarth Load. Please check result on GoogleEarth");
        }

        public void trace(String message)
        {
            Console.WriteLine(message);
            this.Invoke((MethodInvoker)delegate()
            {
                textBoxTrace.Text = message + "\r\n" + textBoxTrace.Text;
            });
        }
    }
}